#!/bin/bash
# Build the FatdogArm basesfs
#
# Copyright (C) James Budiono, 2013, 2014
# License: GPL Version 3 or later
# 
# $1-build option
#    download - download only
#    install - download and install only
#    sfs - download, install and make sfs
#    sfs-pause - same as sfs, but pause after install and before creating sfs
#    (no parameter) - same as "sfs"
#
# $2-config file (default: $HOME/build.conf)
#
# Version 2: Re-write.
# Version 3: Lots of improvements.
#
# Note: must run as root. This script is multiplatform (Fatdog64, FatdogArm)

export LANG=C # make it run a bit faster

### customisable configuration items
PKG_EXT=${PKG_EXT:-tbz} # other choice is tgz or txz
PKG_URL=${PKG_URL:-http://distro.ibiblio.org/fatdog/arm/packages/beta}
PKG_CACHE=${PKG_CACHE:-/tmp/pkgcache}
PKG_LIST=${PKG_LIST:-sfs-pkglist}
GEN_EXCLUSION_LIST=${GEN_EXCLUSION_LIST:-./gen-exclusion-list}

OUTPUT_DIR=${OUTPUT_DIR:-/tmp/output}
KEEP_ROOTFS=${KEEP_ROOTFS:-$ROOTFS_DIR} # if rootfs_dir is specified, keep it (don't delete on exit)
ROOTFS_DIR=${ROOTFS_DIR:-/tmp/rootfs.$$}
INSTALLER_PKG=${INSTALLER_PKG:-pkgtools-14.0-noarch-1.tbz}

BUILD_CONFIG=${2:-$BUILD_CONFIG}
BUILD_CONFIG=${BUILD_CONFIG:-$HOME/build.conf}

### standard configuration items - dont change unless you know what you're doing
[ -e "$BUILD_CONFIG" ] && . "$BUILD_CONFIG"
CHECKSUM_FILE=CHECKSUMS.md5
INSTALLER="$ROOTFS_DIR"/sbin/installpkg

### run-time variables
NUMPKGS=0



################ helpers ###################

# download: $1-file (no path)
download_file() {
	echo downloading "$1" ...
	! [ "${1%/*}" = "${1##*/}" ] && mkdir -p "$PKG_CACHE/${1##*/}" # create path
	wget -c -o /dev/null -O "$PKG_CACHE/$1" "$PKG_URL/$1" 
}

# check file integrity: $1-file (no path)
check_md5() {
	echo -n "Checking integrity $1 ... "
	read goodsum < <(awk -v pkg="./$1" '$2==pkg {print $1;exit}'  "$PKG_CACHE/$CHECKSUM_FILE")
	read filesum < <(md5sum "$PKG_CACHE"/$1 | awk '{print $1}')
	if [ $goodsum != $filesum ]; then
		echo "BAD CHECKSUM !"
		return 1
	fi
	echo good.
	return 0
}



################ action handlers ###############

### cleanup
trap 'cleanup; exit' INT QUIT TERM 0
cleanup() {
	[ -z "$KEEP_ROOTFS" ] && rm -rf "$ROOTFS_DIR"
}

###
setup() {
	# 1. important files must exist
	if ! [ -e "$PKG_LIST" ]; then
		echo Missing components.
		exit
	fi

	# 2. make dirs and get checksum
	mkdir -p "$PKG_CACHE" "$ROOTFS_DIR" "$OUTPUT_DIR"
	! [ -e "$PKG_CACHE/$CHECKSUM_FILE" ] && download_file $CHECKSUM_FILE
	! [ -e "$PKG_CACHE/$CHECKSUM_FILE" ] && echo "Failed to download $CHECKSUM_FILE" && exit
	
	# 3. count number of packages to process
	while read p; do
		case "$p" in
			\#*|"") continue ;;
			*) NUMPKGS=$((NUMPKGS + 1)) ;;
		esac
	done < "$PKG_LIST"
}

###
download() {
	echo downloading packages ...
	PKGS=0
	while read pkg; do
		pkg=${pkg%%#*}; set -- $pkg; pkg=$1
		[ -z $pkg ] && continue;
		PKGS=$((PKGS+1))
		echo -n "$PKGS of $NUMPKGS: "
		if ! [ -e "$PKG_CACHE/$pkg.${PKG_EXT}" ]; then
			download_file $pkg.${PKG_EXT}
		else 
			echo $pkg already exist, not downloaded.
		fi
		if ! check_md5 $pkg.${PKG_EXT}; then
			# bad checksum, attempt re-download
			rm -f "$PKG_CACHE/$pkg.${PKG_EXT}"
			echo Attempting re-download: $pkg
			download_file $pkg.${PKG_EXT}
			check_md5 $pkg.${PKG_EXT}
		fi
	done < "$PKG_LIST"
}

###
installer() {
	echo preparing installer ...
	
	! [ -e "$PKG_CACHE/$INSTALLER_PKG" ] && download_file $INSTALLER_PKG 
	! check_md5 $INSTALLER_PKG && echo "Bad installer package." && exit
	
	tar -xf "$PKG_CACHE"/$INSTALLER_PKG -C "$ROOTFS_DIR"
	rm -rf "$ROOTFS_DIR"/install
	export ROOT="$ROOTFS_DIR"
	if ! [ -e $INSTALLER ]; then
		echo "Can't prepare installer, exiting."
		exit
	fi
}

###
install() {
	# 1. packages
	echo installing packages ...
	PKGS=0
	while read pkg; do
		pkg=${pkg%%#*}; set -- $pkg; pkg=$1
		[ -z $pkg ] && continue
		PKGS=$((PKGS+1))
		
		# don't install if already installed previously
		[ -e "$ROOTFS_DIR/var/log/packages/$pkg" ] && 
		echo $PKGS of $NUMPKGS: "$pkg already installed, skipped..." && continue
		
		echo $PKGS of $NUMPKGS: installing "$pkg" ...
		$INSTALLER "$PKG_CACHE/$pkg.${PKG_EXT}" > /dev/null 2>&1
	done < "$PKG_LIST"
		
	# 2. mark placeholders for exclusion
	#{
	#echo cleaning up...
	#pushd "$ROOTFS_DIR" > /dev/null
	#last-minute clean-up
	#popd > /dev/null
	#} 2>/dev/null	
}

###
makesfs() {
	echo generating exclusion list ...
	echo "#" > "$ROOTFS_DIR"/sfs-exclude
	[ -x $GEN_EXCLUSION_LIST ] && $GEN_EXCLUSION_LIST > "$ROOTFS_DIR"/sfs-exclude
	echo building sfs ...
	mksquashfs "$ROOTFS_DIR" "$OUTPUT_DIR"/fd-arm.sfs -wildcards -noappend \
	            -ef "$ROOTFS_DIR"/sfs-exclude \
	            -e sfs-exclude > /dev/null
}

pause() {
	echo "Press Enter to continue ..."
	read
}

#################### main ######################

# $1-action
case "$1" in
	download)  # download only
		actions="download" ;;
		
	install)   # download and build rootfs
		KEEP_ROOTFS=yes
		actions="download installer install" ;;
		
	sfs|"")    # download, build rootfs, build sfs (=default action)
		actions="download installer install makesfs" ;;
		
	sfs-pause) # same as "sfs" but pause first before building
		actions="download installer install makesfs"
		[ -t 0 ] && actions="download installer install pause makesfs" ;;
		
	help|--help|-h)
		echo "Usage: ${0##*/} [download|install|sfs|sfs-pause]"
		exit
		;;
esac

# 2. always do setup
setup

# 3. run the actions
for action in $actions; do 
	$action
done

# 4. cleanup - executed by trap handler
