#!/bin/dash
# Split and rebuild Slackware packages into base, DEV and NLS packages

export LANG=C # make it run a bit faster

### configuration
WORKDIR=/tmp/splitpkg.$$
SRC_DIR=${SRC_DIR:-output}
OUT_DIR=${OUT_DIR:-split}
SRC_PKG_TYPE=${SRC_PKG_TYPE:-tbz}
OUT_PKG_TYPE=${OUT_PKG_TYPE:-tbz}
LIBDIR=${LIBDIR:-lib}

OUT_DIR=$(readlink -f $OUT_DIR)

trap 'cleanup; exit' INT TERM HUP 0
CLEANED=""
cleanup() {
	[ $CLEANED ] && return; CLEANED=yes
	rm -rf $WORKDIR
}

# $1-full pkg filename (no path), output: PKGNAME VERSION ARCH BUILD
pkgname() {
	local p pp
	PKGNAME="" VERSION="" ARCH="" BUILD=""
	
	pp=${1%.*} # remove extension
	BUILD=${pp##*-};  pp=${pp%-$BUILD}
	ARCH=${pp##*-}; pp=${pp%-$ARCH}
	VERSION=${pp##*-}; PKGNAME=${pp%-$VERSION}
	#echo $PKGNAME $VERSION $ARCH $BUILD
}

# $1-type (nls or dev)
slackdesc() {
	local TEXT OUTDIR LONGTEXT
	case $1 in
		nls) TEXT="NLS" 
			 LONGTEXT="Contains localised messages"
			 OUTDIR=$WORKDIR/nls/install ;;
			 
		dev) TEXT="DEV"
			 LONGTEXT="Development headers and libraries"
			 OUTDIR=$WORKDIR/dev/install ;;
	esac
	
	mkdir -p $OUTDIR
	cat > $OUTDIR/slack-desc << EOF
$PKGNAME-$TEXT: $PKGNAME-$TEXT $VERSION ($TEXT for $PKGNAME)
$PKGNAME-$TEXT:
$PKGNAME-$TEXT: $LONGTEXT for $PKGNAME
$PKGNAME-$TEXT:
$PKGNAME-$TEXT:
$PKGNAME-$TEXT:
$PKGNAME-$TEXT:
$PKGNAME-$TEXT:
$PKGNAME-$TEXT:
$PKGNAME-$TEXT:
$PKGNAME-$TEXT:
EOF
}

# $1-path
remove_empty_dir() {
	find $1 -type d | sort -r | xargs rmdir 2>/dev/null
}

# $1-pkgname (no path)
extract() {
	pkgname $1
	
	rm -rf $WORKDIR # make sure we start with clean slate
	mkdir -p $WORKDIR/base $WORKDIR/dev $WORKDIR/nls
	tar -C $WORKDIR/base -xf $SRC_DIR/$pkg	
}

# $1-type (nls, dev, or base)
package() {
	local OUTFILE
	case $1 in
		nls) OUTFILE=$PKGNAME-NLS-$VERSION-$ARCH-$BUILD.${OUT_PKG_TYPE} ;;
		dev) OUTFILE=$PKGNAME-DEV-$VERSION-$ARCH-$BUILD.${OUT_PKG_TYPE} ;;
		base) OUTFILE=$PKGNAME-$VERSION-$ARCH-$BUILD.${OUT_PKG_TYPE} ;;
	esac
	( 
	  cd $WORKDIR/$1
	  makepkg -c n -l n $OUT_DIR/$OUTFILE > /dev/null 2>&1
	)	
}

# input: BUILD ARCH VERSION PKGNAME
split_nls() {
	# move locales to nls directory
	! [ -e $WORKDIR/base/usr/share/locale ] && return
	mkdir -p $WORKDIR/nls/usr/share
	mv $WORKDIR/base/usr/share/locale $WORKDIR/nls/usr/share
	slackdesc nls
	package nls
}

# input: BUILD ARCH VERSION PKGNAME
split_dev() {
	# recreate dir structure 
	find $WORKDIR/base -type d | sed "s|^$WORKDIR/base||" | while read -r p; do
		[ -z "$p" ] && continue
		mkdir $WORKDIR/dev/$p
	done

	# move includes, docs, src, and pkg-config files
	{ 
	  rm -rf $WORKDIR/dev/usr/include
	  mv $WORKDIR/base/usr/include $WORKDIR/dev/usr
	  
	  rm -rf $WORKDIR/dev/usr/share/doc
	  mv $WORKDIR/base/usr/share/doc $WORKDIR/dev/usr/share
	  
	  rm -rf $WORKDIR/dev/usr/share/gtk-doc
	  mv $WORKDIR/base/usr/share/gtk-doc $WORKDIR/dev/usr/share
	  
	  rm -rf $WORKDIR/dev/usr/$LIBDIR/pkgconfig
	  mv $WORKDIR/base/usr/$LIBDIR/pkgconfig $WORKDIR/dev/usr/$LIBDIR
	  
	  rm -rf $WORKDIR/dev/usr/src
	  mv $WORKDIR/base/usr/src $WORKDIR/dev/usr
	} 2>/dev/null
	
	# static libraries, linker scripts, and M4 files, *-config files
	find $WORKDIR/base -name "*.a" -o -name "*.la" -o -name "*.m4" \
	     -o -name "*-config" \
	     -o -name INSTALL -o -name NEWS -o -name CHANGELOG -o -name README | 
	sed "s|^$WORKDIR/base||" | while read -r p; do
		[ -z "$p" ] && continue
		mv "$WORKDIR/base/$p" "$WORKDIR/dev/$p"
	done
	
	# package - only if not empty
	remove_empty_dir $WORKDIR/dev
	rmdir $WORKDIR/dev 2>/dev/null
	if [ -e $WORKDIR/dev ]; then
		slackdesc dev
		package dev
	fi
}

# input: BUILD ARCH VERSION PKGNAME
rebuild_base() {
	# test if base is now empty
	cp -a $WORKDIR/base $WORKDIR/basecopy
	rm -rf $WORKDIR/basecopy/install # delete package descriptors
	remove_empty_dir $WORKDIR/basecopy
	rmdir $WORKDIR/basecopy 2>/dev/null
	
	# package is base is not totally empty (use original)
	[ -e $WORKDIR/basecopy ] && package base
}

### main
mkdir -p $OUT_DIR/stamp
for f in $SRC_DIR/*.${SRC_PKG_TYPE}; do
	pkg=${f##*/} 
	#pkg=openssl-1.0.1e-armhf-1.tbz
	#pkg=bash-4.2-armhf-1.tbz
	#pkg=util-macros-1.17-armhf-1.tbz
		
	# only process if output is older than source
	if ! [ -e $OUT_DIR/stamp/$pkg ] || [ $OUT_DIR/stamp/$pkg -ot $SRC_DIR/$pkg ]; then
		echo splitting $pkg
			
		# handle exceptions
		case $pkg in
			lfs-tools*|fatdog*) ;; # do nothing
			gcc*|bison*|flex*) 
				extract $pkg
				split_nls
				rebuild_base
				;;
			*)  extract $pkg
				split_nls
				split_dev
				rebuild_base
				;;
		esac
		touch $OUT_DIR/stamp/$pkg
	fi
	#echo please check; read p; break
done
